#include "quicksort.h"

#include <iostream>
#include <new>
#include <cstdlib>
#include <ctime>

using namespace std ;


static void insertNode( Node *&head, int data, bool &ok ) {
    Node *temp = new(nothrow) Node ;
    ok = temp!=0 ;
    if( ok ) {
        temp->data = data ;
        temp->next = head ;
        head = temp ; }
}
        
static void deal( Node *in, Node *&out0, Node *&out1) {
	// We need to do this nonrecursivly to avoid
	// stack overflow on a big list.
	Node **linkP0 = & out0 ;
	Node **linkP1 = & out1 ;
	while( true ) {

		if( in == 0 ) break ;
		*linkP0 = in ;
		linkP0 = & in->next ;
		in = in->next ;

		if( in == 0 ) break ;
		*linkP1 = in ;
		linkP1 = & in->next ;
		in = in->next ; }

	*linkP0 = 0 ;
	*linkP1 = 0 ;
}

static void merge( Node *in0, Node *in1, Node *&out ) {
    // Need to merge nonrecursively
	Node **linkP = & out ;

	while( in0 != 0 || in1 != 0 ) {

		if(in1 == 0 || in0 != 0 && in0->data < in1->data ) {
			*linkP = in0 ;
			linkP = & in0->next ;
			in0 = in0->next ; }
		else {
			*linkP = in1 ;
			linkP = & in1->next ;
			in1 = in1->next ; } }
}

static void mergeSort(Node *&list ) {
    
    if( list != 0 && list->next != 0 ) {
	    Node *p, *q ;
	    
	    // Split the list in to two lists of
	    // roughly equal size.
        deal( list, p, q ) ;
	    
	    //Sort the two sublists.
	    mergeSort(p) ;
	    mergeSort(q) ;
	    
	    //Merge them together.
	    merge(p, q, list) ; }
	    
}

static bool compare( Node *p, Node *q ) {
	while( p!=0 && q!=0 ) {
		if( p->data != q->data ) return false ;
		p = p->next ;
		q = q->next ; }
	if( p!=0 || q!=0 ) return false ;
	else return true ;
}

static void copy( Node *head, Node *&newHead, bool &ok ) {
	
	Node *p = head ;
	Node **linkP = &newHead ;
	
	// Loop invariant: 
	//   * p has the same value as one of the links of
	//     the list headed by head,
	//   * linkP points to the last link in the
	//     list headed by newHead,
	//   * *linkP has not been initialized, and
	//   * between newHead and *linkP are copies
	//     of all the nodes between head and p. 
    while( p != 0 ) {
		// Allocate a new node
			Node *temp = new(nothrow) Node ;
			if( temp==0 ) { *linkP = 0 ; ok = false ; return ; }
		// Copy the data field.
			temp->data = p->data ;
		// Add it to the new list
			*linkP = temp ;
		// Advance p and linkP
			p = p->next ;
			linkP = & (*linkP)->next ; }
	// Set the tail to null
	    *linkP = 0 ;
}

void print(Node *p) {
	cout << "<" ;
	while( p != 0 ) {
		cout << p->data ;
		p = p->next ;
		if( p != 0 ) cout << ", " ; }
	cout <<">" << endl ;
}

static void delList( Node *&head ) {
	while( head ) {
		Node *temp = head ;
		head = head->next ;
		delete temp ; }
}

static void testRand( int n, bool &pass, void (*func)(Node*&) ) {
	bool ok ;
	Node *head = 0 ;
	srand( 2004 ) ;
	for( int i=0 ; i<n ; ++i ) {
		insertNode( head, rand(), ok ) ; }
	Node * check ; copy( head, check, ok ) ;
	clock_t t0 = clock() ;
	mergeSort( check ) ;
	clock_t t1 = clock() ;
	(*func)( head ) ;
	clock_t t2 = clock() ;
	double delta_merge = 1000.0 * (double)(t1-t0) / CLOCKS_PER_SEC ;
	double delta_quick = 1000.0 * (double)(t2-t1) / CLOCKS_PER_SEC ;
	cout << "merge sort: " << delta_merge << "ms" <<endl ;
	cout << "quick sort: " << delta_quick << "ms" <<endl ;
	//cout<< "Expected: " ; print( check ) ;
	//cout<< "Result:   " ; print( head ) ;
	pass = compare( head, check ) ;
	delList( head ) ;
	delList( check ) ;
}

int testCount() { return 5 ; }

// doTest  --- do one test //
/////////////////////////////
void doTest( int testNumber, int &result ) {
    bool pass = false ;
    try {
        cout << "Test " << testNumber << "\n" ;
        switch( testNumber ) {
        case 0 : {
			Node *head = 0 ;
			quicksort( head ) ;
			pass = compare(head, 0 ) ;
			cout<< "Expected: " ; print( 0 ) ;
			cout<< "Result:   " ; print( head ) ;
		} break ;
        case 1 : {
			bool ok ;
			Node *head = 0 ;
			insertNode( head, 60, ok ) ;
			Node * check ; copy( head, check, ok ) ;
			mergeSort( check ) ;
			quicksort( head ) ;
			cout<< "Expected: " ; print( check ) ;
			cout<< "Result:   " ; print( head ) ;
			pass = compare( head, check ) ;
			delList( head ) ;
			delList( check ) ;
		} break ;
        case 2 : {
			bool ok ;
			Node *head = 0 ;
			insertNode( head, 60, ok ) ;
			insertNode( head, 70, ok ) ;
			insertNode( head, 20, ok ) ;
			insertNode( head, 10, ok ) ;
			insertNode( head, 50, ok ) ;
			insertNode( head, 90, ok ) ;
			Node * check ; copy( head, check, ok ) ;
			mergeSort( check ) ;
			quicksort( head ) ;
			cout<< "Expected: " ; print( check ) ;
			cout<< "Result:   " ; print( head ) ;
			pass = compare( head, check ) ;
			delList( head ) ;
			delList( check ) ;
		} break ;
        case 3 : {
			testRand( 100, pass, &quicksort ) ;
		} break ;
        case 4 : {
			// If you are getting a stack overflow
			// exception on this test, make sure that
			// you are not using any recursion that
			// will cause the stack depth to equal
			// the length of the list.
			testRand( 100000, pass, &optimizedQuicksort ) ;
			
		} break ;
        default : pass = false ;
        } }
    catch( ... ) {
        cerr << "Sorry an unanticipated exception occurred\n" ;
        pass = false ; }
	

    if( pass ) {
        cout << "Test " << testNumber << " passes\n" ;
        result = 0 ; }
    else {
        cout << "Test " << testNumber << " fails\n" ;
        result = 1 ; }
}